import { ref, unref, watch, watchEffect } from 'vue';
import useState from './useState';
export default function useMergedState(defaultStateValue, option) {
    const { defaultValue, value = ref() } = option || {};
    let initValue = typeof defaultStateValue === 'function' ? defaultStateValue() : defaultStateValue;
    if (value.value !== undefined) {
        initValue = unref(value);
    }
    if (defaultValue !== undefined) {
        initValue = typeof defaultValue === 'function' ? defaultValue() : defaultValue;
    }
    const innerValue = ref(initValue);
    const mergedValue = ref(initValue);
    const [prevValue, setPrevValue] = useState([mergedValue.value]);
    watchEffect(() => {
        let val = value.value !== undefined ? value.value : innerValue.value;
        if (option === null || option === void 0 ? void 0 : option.postState) {
            val = option.postState(val, innerValue.value);
        }
        mergedValue.value = val;
    });
    watch(prevValue, () => {
        var _a;
        const prev = prevValue.value[0];
        if (innerValue.value !== prev) {
            (_a = option === null || option === void 0 ? void 0 : option.onChange) === null || _a === void 0 ? void 0 : _a.call(option, innerValue.value, prev);
        }
    }, { deep: true });
    function setInnerValue(newValue) {
        innerValue.value = newValue;
    }
    function triggerChange(newValue) {
        setInnerValue(newValue);
        setPrevValue([mergedValue.value]);
    }
    // Effect of reset value to `undefined`
    watch(value, () => {
        innerValue.value = value.value;
    });
    return [mergedValue, triggerChange, innerValue];
}
