"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = useMergedState;
const vue_1 = require("vue");
const useState_1 = require("./useState");
function useMergedState(defaultStateValue, option) {
    const { defaultValue, value = (0, vue_1.ref)() } = option || {};
    let initValue = typeof defaultStateValue === 'function' ? defaultStateValue() : defaultStateValue;
    if (value.value !== undefined) {
        initValue = (0, vue_1.unref)(value);
    }
    if (defaultValue !== undefined) {
        initValue = typeof defaultValue === 'function' ? defaultValue() : defaultValue;
    }
    const innerValue = (0, vue_1.ref)(initValue);
    const mergedValue = (0, vue_1.ref)(initValue);
    const [prevValue, setPrevValue] = (0, useState_1.default)([mergedValue.value]);
    (0, vue_1.watchEffect)(() => {
        let val = value.value !== undefined ? value.value : innerValue.value;
        if (option === null || option === void 0 ? void 0 : option.postState) {
            val = option.postState(val, innerValue.value);
        }
        mergedValue.value = val;
    });
    (0, vue_1.watch)(prevValue, () => {
        var _a;
        const prev = prevValue.value[0];
        if (innerValue.value !== prev) {
            (_a = option === null || option === void 0 ? void 0 : option.onChange) === null || _a === void 0 ? void 0 : _a.call(option, innerValue.value, prev);
        }
    }, { deep: true });
    function setInnerValue(newValue) {
        innerValue.value = newValue;
    }
    function triggerChange(newValue) {
        setInnerValue(newValue);
        setPrevValue([mergedValue.value]);
    }
    // Effect of reset value to `undefined`
    (0, vue_1.watch)(value, () => {
        innerValue.value = value.value;
    });
    return [mergedValue, triggerChange, innerValue];
}
