"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryOnUnmounted = tryOnUnmounted;
exports.default = useTimeoutFn;
exports.useTimeoutRef = useTimeoutRef;
const vue_1 = require("vue");
const pro_utils_1 = require("@gx-design-vue/pro-utils");
function tryOnUnmounted(fn) {
    if ((0, vue_1.getCurrentInstance)())
        (0, vue_1.onUnmounted)(fn);
}
function useTimeoutFn(handle, wait, native = false) {
    if (!(0, pro_utils_1.isFunction)(handle)) {
        throw new Error('handle is not Function!');
    }
    const { readyRef, stop, start } = useTimeoutRef(wait);
    if (native) {
        handle();
    }
    else {
        (0, vue_1.watch)(readyRef, (maturity) => {
            // eslint-disable-next-line ts/no-unused-expressions
            maturity && handle();
        }, { immediate: false });
    }
    return { readyRef, stop, start };
}
function useTimeoutRef(wait) {
    const readyRef = (0, vue_1.ref)(false);
    let timer;
    function stop() {
        readyRef.value = false;
        timer && window.clearTimeout(timer);
    }
    function start() {
        stop();
        timer = setTimeout(() => {
            readyRef.value = true;
        }, wait);
    }
    start();
    tryOnUnmounted(stop);
    return { readyRef, stop, start };
}
